@@ -15,7 +15,7 @@ Stanza Commands Handler
 
 /**********************************************************************************************************************************/
 FN_EXTERN String *
-cipherPassGen(CipherType cipherType)
+cipherPassGen(const CipherType cipherType)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(ENUM, cipherType);
@@ -47,7 +47,7 @@ pgValidate(void)
         if (cfgOptionBool(cfgOptOnline))
         {
             // Check the primary connections (and standby, if any) and return the primary database object.
-            DbGetResult dbObject = dbGet(false, true, false);
+            const DbGetResult dbObject = dbGet(false, true, CFGOPTVAL_BACKUP_STANDBY_N);
 
             // Get the pgControl information from the pg*-path deemed to be the primary
             result = dbPgControl(dbObject.primary);
@@ -57,7 +57,7 @@ pgValidate(void)
         }
         // If the database is not online, assume that pg1 is the primary
         else
-            result = pgControlFromFile(storagePg());
+            result = pgControlFromFile(storagePg(), cfgOptionStrNull(cfgOptPgVersionForce));
     }
     MEM_CONTEXT_TEMP_END();
 
