@@ -5,6 +5,7 @@ Protocol Helper
 
 #include <string.h>
 
+#include "command/lock.h"
 #include "common/crypto/common.h"
 #include "common/debug.h"
 #include "common/exec.h"
@@ -74,8 +75,8 @@ protocolHelperInit(void)
 }
 
 /**********************************************************************************************************************************/
-bool
-repoIsLocal(unsigned int repoIdx)
+FN_EXTERN bool
+repoIsLocal(const unsigned int repoIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(UINT, repoIdx);
@@ -85,7 +86,7 @@ repoIsLocal(unsigned int repoIdx)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 repoIsLocalVerify(void)
 {
     FUNCTION_TEST_VOID();
@@ -96,8 +97,8 @@ repoIsLocalVerify(void)
 }
 
 /**********************************************************************************************************************************/
-void
-repoIsLocalVerifyIdx(unsigned int repoIdx)
+FN_EXTERN void
+repoIsLocalVerifyIdx(const unsigned int repoIdx)
 {
     FUNCTION_TEST_VOID();
 
@@ -108,8 +109,8 @@ repoIsLocalVerifyIdx(unsigned int repoIdx)
 }
 
 /**********************************************************************************************************************************/
-bool
-pgIsLocal(unsigned int pgIdx)
+FN_EXTERN bool
+pgIsLocal(const unsigned int pgIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(UINT, pgIdx);
@@ -119,7 +120,7 @@ pgIsLocal(unsigned int pgIdx)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 pgIsLocalVerify(void)
 {
     FUNCTION_TEST_VOID();
@@ -134,7 +135,7 @@ pgIsLocalVerify(void)
 Get the command line required for local protocol execution
 ***********************************************************************************************************************************/
 static StringList *
-protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostIdx, unsigned int processId)
+protocolLocalParam(const ProtocolStorageType protocolStorageType, const unsigned int hostIdx, const unsigned int processId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING_ID, protocolStorageType);
@@ -147,7 +148,7 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostIdx
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Option replacements
-        KeyValue *optionReplace = kvNew();
+        KeyValue *const optionReplace = kvNew();
 
         // Add the process id -- used when more than one process will be called
         kvPut(optionReplace, VARSTRDEF(CFGOPT_PROCESS), VARUINT(processId));
@@ -186,7 +187,8 @@ protocolLocalParam(ProtocolStorageType protocolStorageType, unsigned int hostIdx
 // Helper to execute the local process. This is a separate function solely so that it can be shimmed during testing.
 static void
 protocolLocalExec(
-    ProtocolHelperClient *helper, ProtocolStorageType protocolStorageType, unsigned int hostIdx, unsigned int processId)
+    ProtocolHelperClient *const helper, const ProtocolStorageType protocolStorageType, const unsigned int hostIdx,
+    const unsigned int processId)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, helper);
@@ -195,6 +197,8 @@ protocolLocalExec(
         FUNCTION_TEST_PARAM(UINT, processId);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(helper != NULL);
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -229,8 +233,8 @@ protocolLocalExec(
     FUNCTION_TEST_RETURN_VOID();
 }
 
-ProtocolClient *
-protocolLocalGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx, unsigned int processId)
+FN_EXTERN ProtocolClient *
+protocolLocalGet(const ProtocolStorageType protocolStorageType, const unsigned int hostIdx, const unsigned int processId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING_ID, protocolStorageType);
@@ -279,7 +283,7 @@ Free the protocol client and underlying exec'd process. Log any errors as warnin
 while closing a local/remote that has already completed its work. The warning will be an indication that something is not right.
 ***********************************************************************************************************************************/
 static void
-protocolHelperClientFree(ProtocolHelperClient *protocolHelperClient)
+protocolHelperClientFree(ProtocolHelperClient *const protocolHelperClient)
 {
     FUNCTION_LOG_BEGIN(logLevelTrace);
         FUNCTION_LOG_PARAM_P(VOID, protocolHelperClient);
@@ -323,8 +327,8 @@ protocolHelperClientFree(ProtocolHelperClient *protocolHelperClient)
 }
 
 /**********************************************************************************************************************************/
-void
-protocolLocalFree(unsigned int processId)
+FN_EXTERN void
+protocolLocalFree(const unsigned int processId)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(UINT, processId);
@@ -386,7 +390,7 @@ protocolServerAuthorize(const String *authListStr, const String *const stanza)
     FUNCTION_TEST_RETURN(BOOL, result);
 }
 
-ProtocolServer *
+FN_EXTERN ProtocolServer *
 protocolServer(IoServer *const tlsServer, IoSession *const socketSession)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -410,6 +414,11 @@ protocolServer(IoServer *const tlsServer, IoSession *const socketSession)
         {
             TRY_BEGIN()
             {
+                // Get parameter list from the client. This needs to happen even if we cannot authorize the client so that the
+                // session id gets set for the error.
+                const ProtocolServerRequestResult command = protocolServerRequest(result);
+                CHECK(FormatError, command.id == PROTOCOL_COMMAND_CONFIG, "expected config command");
+
                 // Get list of authorized stanzas for this client
                 CHECK(AssertError, cfgOptionTest(cfgOptTlsServerAuth), "missing auth data");
 
@@ -420,10 +429,7 @@ protocolServer(IoServer *const tlsServer, IoSession *const socketSession)
                 if (clientAuthList == NULL)
                     THROW(AccessError, "access denied");
 
-                // Get parameter list from the client and load it
-                const ProtocolServerCommandGetResult command = protocolServerCommandGet(result);
-                CHECK(FormatError, command.id == PROTOCOL_COMMAND_CONFIG, "expected config command");
-
+                // Load parameter list from the client
                 StringList *const paramList = pckReadStrLstP(pckReadNew(command.param));
                 strLstInsert(paramList, 0, cfgExe());
                 cfgLoad(strLstSize(paramList), strLstPtr(paramList));
@@ -440,16 +446,21 @@ protocolServer(IoServer *const tlsServer, IoSession *const socketSession)
             TRY_END();
 
             // Ack the config command
-            protocolServerDataEndPut(result);
+            protocolServerResponseP(result);
 
-            ioSessionMove(tlsSession, memContextPrior());
+            // Move result to prior context and move session into result so there is only one return value
             protocolServerMove(result, memContextPrior());
+            ioSessionMove(tlsSession, objMemContext(result));
         }
         // Else the client can only detect that the server is alive
         else
         {
-            // Send a data end message and return a NULL server. Do not waste time looking at what the client wrote.
-            protocolServerDataEndPut(result);
+            // A noop command should have been received
+            const ProtocolServerRequestResult command = protocolServerRequest(result);
+            CHECK(FormatError, command.id == PROTOCOL_COMMAND_NOOP, "expected config command");
+
+            // Send a data end message and return a NULL server
+            protocolServerResponseP(result);
 
             // Set result to NULL so there is no server for the caller to use. The TLS session will be freed when the temp mem
             // context ends.
@@ -465,7 +476,7 @@ protocolServer(IoServer *const tlsServer, IoSession *const socketSession)
 Get the command line required for remote protocol execution
 ***********************************************************************************************************************************/
 static StringList *
-protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
+protocolRemoteParam(const ProtocolStorageType protocolStorageType, const unsigned int hostIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING_ID, protocolStorageType);
@@ -477,30 +488,31 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int hostId
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Is this a repo remote?
-        bool isRepo = protocolStorageType == protocolStorageTypeRepo;
+        const bool isRepo = protocolStorageType == protocolStorageTypeRepo;
 
         // Option replacements
-        KeyValue *optionReplace = kvNew();
+        KeyValue *const optionReplace = kvNew();
 
         // Replace config options with the host versions
-        unsigned int optConfig = isRepo ? cfgOptRepoHostConfig : cfgOptPgHostConfig;
+        const unsigned int optConfig = isRepo ? cfgOptRepoHostConfig : cfgOptPgHostConfig;
 
         kvPut(
             optionReplace, VARSTRDEF(CFGOPT_CONFIG),
             cfgOptionIdxSource(optConfig, hostIdx) != cfgSourceDefault ? VARSTR(cfgOptionIdxStr(optConfig, hostIdx)) : NULL);
 
-        unsigned int optConfigIncludePath = isRepo ? cfgOptRepoHostConfigIncludePath : cfgOptPgHostConfigIncludePath;
+        const unsigned int optConfigIncludePath = isRepo ? cfgOptRepoHostConfigIncludePath : cfgOptPgHostConfigIncludePath;
 
         kvPut(
             optionReplace, VARSTRDEF(CFGOPT_CONFIG_INCLUDE_PATH),
             cfgOptionIdxSource(optConfigIncludePath, hostIdx) != cfgSourceDefault ?
                 VARSTR(cfgOptionIdxStr(optConfigIncludePath, hostIdx)) : NULL);
 
-        unsigned int optConfigPath = isRepo ? cfgOptRepoHostConfigPath : cfgOptPgHostConfigPath;
+        const unsigned int optConfigPath = isRepo ? cfgOptRepoHostConfigPath : cfgOptPgHostConfigPath;
 
         kvPut(
             optionReplace, VARSTRDEF(CFGOPT_CONFIG_PATH),
-            cfgOptionIdxSource(optConfigPath, hostIdx) != cfgSourceDefault ? VARSTR(cfgOptionIdxStr(optConfigPath, hostIdx)) : NULL);
+            cfgOptionIdxSource(optConfigPath, hostIdx) != cfgSourceDefault ?
+                VARSTR(cfgOptionIdxStr(optConfigPath, hostIdx)) : NULL);
 
         // Update/remove repo/pg options that are sent to the remote
         for (ConfigOption optionId = 0; optionId < CFG_OPTION_TOTAL; optionId++)
@@ -539,7 +551,9 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int hostId
                 // Remove unrequired/defaulted pg options when the remote type is repo since they won't be used
                 if (protocolStorageType == protocolStorageTypeRepo)
                 {
-                    remove = !cfgParseOptionRequired(cfgCommand(), optionId) || cfgParseOptionDefault(cfgCommand(), optionId) != NULL;
+                    remove =
+                        !cfgParseOptionRequired(cfgCommand(), optionId) ||
+                        cfgParseOptionDefault(cfgCommand(), optionId) != NULL;
                 }
                 // Move pg options to host index 0 (key 1) so they will be in the default index on the remote host
                 else
@@ -556,8 +570,8 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int hostId
                 }
             }
 
-            // Remove options that have been marked for removal if they are not already null or invalid. This is more efficient because
-            // cfgExecParam() won't have to search through as large a list looking for overrides.
+            // Remove options that have been marked for removal if they are not already null or invalid. This is more efficient
+            // because cfgExecParam() won't have to search through as large a list looking for overrides.
             if (remove)
             {
                 // Loop through option indexes
@@ -596,6 +610,10 @@ protocolRemoteParam(ProtocolStorageType protocolStorageType, unsigned int hostId
         // Add the remote type
         kvPut(optionReplace, VARSTRDEF(CFGOPT_REMOTE_TYPE), VARSTR(strIdToStr(protocolStorageType)));
 
+        // Add lock required on the remote
+        if (cfgLockRemoteRequired())
+            kvPut(optionReplace, VARSTRDEF(CFGOPT_LOCK), varNewVarLst(varLstNewStrLst(cmdLockList())));
+
         MEM_CONTEXT_PRIOR_BEGIN()
         {
             result = cfgExecParam(cfgCommand(), cfgCmdRoleRemote, optionReplace, false, true);
@@ -616,12 +634,12 @@ protocolRemoteParamSsh(const ProtocolStorageType protocolStorageType, const unsi
         FUNCTION_LOG_PARAM(UINT, hostIdx);
     FUNCTION_LOG_END();
 
-    StringList *result = strLstNew();
+    StringList *const result = strLstNew();
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
         // Is this a repo remote?
-        bool isRepo = protocolStorageType == protocolStorageTypeRepo;
+        const bool isRepo = protocolStorageType == protocolStorageTypeRepo;
 
         // Fixed parameters for ssh command
         strLstAddZ(result, "-o");
@@ -632,7 +650,7 @@ protocolRemoteParamSsh(const ProtocolStorageType protocolStorageType, const unsi
         strLstAddZ(result, "PasswordAuthentication=no");
 
         // Append port if specified
-        ConfigOption optHostPort = isRepo ? cfgOptRepoHostPort : cfgOptPgHostPort;
+        const ConfigOption optHostPort = isRepo ? cfgOptRepoHostPort : cfgOptPgHostPort;
 
         if (cfgOptionIdxTest(optHostPort, hostIdx))
         {
@@ -646,7 +664,7 @@ protocolRemoteParamSsh(const ProtocolStorageType protocolStorageType, const unsi
             strZ(cfgOptionIdxStr(isRepo ? cfgOptRepoHost : cfgOptPgHost, hostIdx)));
 
         // Add remote command and parameters
-        StringList *paramList = protocolRemoteParam(protocolStorageType, hostIdx);
+        StringList *const paramList = protocolRemoteParam(protocolStorageType, hostIdx);
 
         strLstInsert(paramList, 0, cfgOptionIdxStr(isRepo ? cfgOptRepoHostCmd : cfgOptPgHostCmd, hostIdx));
         strLstAdd(result, strLstJoin(paramList, " "));
@@ -670,6 +688,8 @@ protocolRemoteExec(
         FUNCTION_TEST_PARAM(UINT, processId);
     FUNCTION_TEST_END();
 
+    FUNCTION_AUDIT_HELPER();
+
     ASSERT(helper != NULL);
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -759,11 +779,23 @@ protocolRemoteExec(
             {
                 ASSERT(remoteType == CFGOPTVAL_REPO_HOST_TYPE_TLS);
 
-                // Pass parameters to server
-                ProtocolCommand *const command = protocolCommandNew(PROTOCOL_COMMAND_CONFIG);
-                pckWriteStrLstP(protocolCommandParam(command), protocolRemoteParam(protocolStorageType, hostIdx));
-                protocolClientExecute(helper->client, command, false);
-                protocolCommandFree(command);
+                TRY_BEGIN()
+                {
+                    // Pass parameters to server
+                    PackWrite *const param = protocolPackNew();
+
+                    pckWriteStrLstP(param, protocolRemoteParam(protocolStorageType, hostIdx));
+                    protocolClientRequestP(helper->client, PROTOCOL_COMMAND_CONFIG, .param = param);
+                }
+                CATCH_ANY()
+                {
+                    // Clear the callback so the client does not try to shut down the connection. Attempting to shut down the
+                    // connection will fail since the server has already disconnected and a new error will be thrown, masking the
+                    // original error.
+                    memContextCallbackClear(objMemContext(helper->client));
+                    RETHROW();
+                }
+                TRY_END();
 
                 break;
             }
@@ -774,8 +806,8 @@ protocolRemoteExec(
     FUNCTION_TEST_RETURN_VOID();
 }
 
-ProtocolClient *
-protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
+FN_EXTERN ProtocolClient *
+protocolRemoteGet(const ProtocolStorageType protocolStorageType, const unsigned int hostIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING_ID, protocolStorageType);
@@ -783,7 +815,7 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
     FUNCTION_LOG_END();
 
     // Is this a repo remote?
-    bool isRepo = protocolStorageType == protocolStorageTypeRepo;
+    const bool isRepo = protocolStorageType == protocolStorageTypeRepo;
 
     protocolHelperInit();
 
@@ -801,18 +833,14 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
         MEM_CONTEXT_END();
     }
 
-    // Determine protocol id for the remote.  If the process option is set then use that since we want the remote protocol id to
+    // Determine protocol id for the remote. If the process option is set then use that since we want the remote protocol id to
     // match the local protocol id. Otherwise set to 0 since the remote is being started from a main process and there should only
     // be one remote per host.
-    unsigned int processId = 0;
-
-    if (cfgOptionTest(cfgOptProcess))
-        processId = cfgOptionUInt(cfgOptProcess);
-
+    const unsigned int processId = cfgOptionTest(cfgOptProcess) ? cfgOptionUInt(cfgOptProcess) : 0;
     CHECK(AssertError, hostIdx < protocolHelper.clientRemoteSize, "invalid host");
 
     // Create protocol object
-    ProtocolHelperClient *protocolHelperClient = &protocolHelper.clientRemote[hostIdx];
+    ProtocolHelperClient *const protocolHelperClient = &protocolHelper.clientRemote[hostIdx];
 
     if (protocolHelperClient->client == NULL)
     {
@@ -827,11 +855,11 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
             if (isRepo && cfgOptionIdxStrId(cfgOptRepoCipherType, hostIdx) == cipherTypeNone)
             {
                 // Options to query
-                VariantList *param = varLstNew();
+                VariantList *const param = varLstNew();
                 varLstAdd(param, varNewStrZ(cfgOptionIdxName(cfgOptRepoCipherType, hostIdx)));
                 varLstAdd(param, varNewStrZ(cfgOptionIdxName(cfgOptRepoCipherPass, hostIdx)));
 
-                VariantList *optionList = configOptionRemote(protocolHelperClient->client, param);
+                const VariantList *const optionList = configOptionRemote(protocolHelperClient->client, param);
 
                 if (varUInt64(varLstGet(optionList, 0)) != cipherTypeNone)
                 {
@@ -847,8 +875,8 @@ protocolRemoteGet(ProtocolStorageType protocolStorageType, unsigned int hostIdx)
 }
 
 /**********************************************************************************************************************************/
-void
-protocolRemoteFree(unsigned int hostIdx)
+FN_EXTERN void
+protocolRemoteFree(const unsigned int hostIdx)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(UINT, hostIdx);
@@ -861,7 +889,7 @@ protocolRemoteFree(unsigned int hostIdx)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 protocolKeepAlive(void)
 {
     FUNCTION_LOG_VOID(logLevelTrace);
@@ -879,7 +907,7 @@ protocolKeepAlive(void)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 protocolFree(void)
 {
     FUNCTION_LOG_VOID(logLevelTrace);
