@@ -24,10 +24,10 @@ static HrnHostTestDefine testMatrix[] =
     {.pg =  "10", .repo =  "pg2", .tls = 0, .stg =  "sftp", .enc = 1, .cmp =   "gz", .rt = 1, .bnd = 1, .bi = 0},
     {.pg =  "11", .repo = "repo", .tls = 1, .stg =   "gcs", .enc = 0, .cmp =  "zst", .rt = 2, .bnd = 0, .bi = 0},
     {.pg =  "12", .repo = "repo", .tls = 0, .stg =    "s3", .enc = 1, .cmp =  "lz4", .rt = 1, .bnd = 1, .bi = 1},
-    {.pg =  "13", .repo =  "pg2", .tls = 1, .stg =  "sftp", .enc = 0, .cmp =  "zst", .rt = 1, .bnd = 1, .bi = 1},
+    {.pg =  "13", .repo =  "pg2", .tls = 1, .stg = "posix", .enc = 0, .cmp = "none", .rt = 1, .bnd = 0, .bi = 0},
     {.pg =  "14", .repo = "repo", .tls = 0, .stg =   "gcs", .enc = 0, .cmp =  "lz4", .rt = 1, .bnd = 1, .bi = 0},
     {.pg =  "15", .repo =  "pg2", .tls = 0, .stg = "azure", .enc = 0, .cmp = "none", .rt = 2, .bnd = 1, .bi = 1},
-    {.pg =  "16", .repo = "repo", .tls = 0, .stg = "posix", .enc = 0, .cmp = "none", .rt = 1, .bnd = 0, .bi = 0},
+    {.pg =  "16", .repo = "repo", .tls = 0, .stg =  "sftp", .enc = 0, .cmp =  "zst", .rt = 1, .bnd = 1, .bi = 1},
     {.pg =  "17", .repo = "repo", .tls = 0, .stg = "posix", .enc = 0, .cmp = "none", .rt = 1, .bnd = 0, .bi = 0},
     // {uncrustify_on}
 };
@@ -355,7 +355,7 @@ testRun(void)
             HRN_HOST_WAL_SWITCH(pg1);
         }
 
-        // Store the time so it can be used in a later test
+        // Store the timeline so it can be used in a later test
         const char *const xidTimeline = strZ(
             pckReadStrP(
                 hrnHostSqlValue(
@@ -381,6 +381,11 @@ testRun(void)
             TEST_HOST_SQL_ONE_STR_Z(pg1, "select message from status", TEST_STATUS_TIME);
         }
 
+        // Store the time where the entire timeline captured in xidTimeline above exists so it can be used in a later test. We need
+        // to capture this here (instead of above) to be sure that all the timeline WAL has been archived.
+        const char *const xidTime = strZ(pckReadStrP(hrnHostSqlValue(pg1, "select current_timestamp::text")));
+        TEST_LOG_FMT("xid time = %s", xidTime);
+
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("primary restore (time xid, exclusive)");
         {
@@ -420,7 +425,21 @@ testRun(void)
             // Stop the cluster
             HRN_HOST_PG_STOP(pg1);
 
-            TEST_HOST_BR(pg1, CFGCMD_RESTORE, .option = zNewFmt("--delta --type=standby --target-timeline=%s", xidTimeline));
+            // If repo is versioned then delete the repo to test repo-target-time
+            if (hrnHostRepoVersioning())
+            {
+                // Stop pgbackrest
+                TEST_HOST_BR(repo, CFGCMD_STOP);
+
+                // Delete stanza
+                TEST_HOST_BR(repo, CFGCMD_STANZA_DELETE);
+            }
+
+            TEST_HOST_BR(
+                pg1, CFGCMD_RESTORE,
+                .option = zNewFmt(
+                    "--delta --type=standby --target-timeline=%s%s", xidTimeline,
+                    hrnHostRepoVersioning() ? zNewFmt(" --repo=1 --repo-target-time=\"%s\"", xidTime) : ""));
             HRN_HOST_PG_START(pg1);
 
             // Check that backup recovered to the expected target
@@ -428,7 +447,7 @@ testRun(void)
         }
 
         // -------------------------------------------------------------------------------------------------------------------------
-        if (hrnHostNonVersionSpecific())
+        if (!hrnHostRepoVersioning() && hrnHostNonVersionSpecific())
         {
             TEST_TITLE("stanza-delete --force with pgbackrest stopped");
 
